﻿#region License Informtion
/* This program simulates objects path of motion.
 * Copyright (C) 2010  Timothy Eck II
 * 
 * Physics Simulator is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; version 2 of the License.
 * 
 * Physics Simulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PhysicsSimulator.Engine
{
    /// <summary>
    /// A struct defining the terms to end a simulation
    /// </summary>
    struct KillCondition
    {
        private string _nameOfObjectBeingMonitored;
        
        private bool _endAtVector;
        private Vector2 _endVector;
        private bool _endAtVelocity;
        private Vector2 _endVelocity;

        private bool _endAtYCoord;
        private double _endYCoord;
        private bool _endAtXCoord;
        private double _endXCoord;

        private bool _endAfterRealWorldTimeElapse;
        private TimeSpan _realWorldElapse;
        private bool _endAfterSimulationElapsedTime;
        private TimeSpan _simulationElapsedTime;


        #region Properties
        /// <summary>
        /// Gets or sets the name of object being monitored. All the kill conditions in this struct will be based off this object.
        /// </summary>
        /// <value>The name of object being monitored.</value>
        public string NameOfObjectBeingMonitored
        {
            get { return _nameOfObjectBeingMonitored; }
            set { _nameOfObjectBeingMonitored = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to [end at a vector].
        /// </summary>
        /// <value>[end at a specified vector] if <c>true</c>; otherwise, <c>false</c>.</value>
        public bool EndAtVector
        {
            get { return _endAtVector; }
            set { _endAtVector = value; }
        }

        /// <summary>
        /// Gets or sets the end vector to end the simulation at.
        /// </summary>
        /// <value>The end vector.</value>
        public Vector2 EndVector
        {
            get { return _endVector; }
            set { _endVector = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to [end at a  certain velocity].
        /// </summary>
        /// <value>[end at a certain velocity] if <c>true</c>; otherwise, <c>false</c>.</value>
        public bool EndAtVelocity
        {
            get { return _endAtVelocity; }
            set { _endAtVelocity = value; }
        }

        /// <summary>
        /// Gets or sets the velocity to end at.
        /// </summary>
        /// <value>The end velocity.</value>
        public Vector2 EndVelocity
        {
            get { return _endVelocity; }
            set { _endVelocity = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not to [end at Y coord].
        /// </summary>
        /// <value><c>true</c> if [end at Y coord]; otherwise, <c>false</c>.</value>
        public bool EndAtYCoord
        {
            get { return _endAtYCoord; }
            set { _endAtYCoord = value; }
        }

        /// <summary>
        /// Gets or sets the end Y coord to end at.
        /// </summary>
        /// <value>The end Y coord.</value>
        public double EndYCoord
        {
            get { return _endYCoord; }
            set { _endYCoord = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not to [end at X coord].
        /// </summary>
        /// <value><c>true</c> if [end at X coord]; otherwise, <c>false</c>.</value>
        public bool EndAtXCoord
        {
            get { return _endAtXCoord; }
            set { _endAtXCoord = value; }
        }

        /// <summary>
        /// Gets or sets the end X coord to end at.
        /// </summary>
        /// <value>The end X coord.</value>
        public double EndXCoord
        {
            get { return _endXCoord; }
            set { _endXCoord = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to [end after real world time elapse].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [end after real world time elapse]; otherwise, <c>false</c>.
        /// </value>
        public bool EndAfterRealWorldTimeElapse
        {
            get { return _endAfterRealWorldTimeElapse; }
            set { _endAfterRealWorldTimeElapse = value; }
        }

        /// <summary>
        /// Gets or sets the real world elapse time.
        /// </summary>
        /// <value>The real world elapse time.</value>
        public TimeSpan RealWorldElapse
        {
            get { return _realWorldElapse; }
            set { _realWorldElapse = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to [end after simulation elapsed time].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [end after simulation elapsed time]; otherwise, <c>false</c>.
        /// </value>
        public bool EndAfterSimulationElapsedTime
        {
            get { return _endAfterSimulationElapsedTime; }
            set { _endAfterSimulationElapsedTime = value; }
        }

        /// <summary>
        /// Gets or sets the simulation elapsed time.
        /// </summary>
        /// <value>The simulation elapsed time.</value>
        public TimeSpan SimulationElapsedTime
        {
            get { return _simulationElapsedTime; }
            set { _simulationElapsedTime = value; }
        }
        #endregion

    }
}
