﻿#region License Informtion
/* This program simulates objects path of motion.
 * Copyright (C) 2010  Timothy Eck II
 * 
 * Physics Simulator is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; version 2 of the License.
 * 
 * Physics Simulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
#endregion
using System;
using System.Collections.Generic;
using System.Threading;
using PhysicsSimulator.Reports;

namespace PhysicsSimulator.Engine
{
    /// <summary>
    /// Contains the code to run the simulations.
    /// </summary>
    class Simulator
    {
        private static Thread simThread;
        
        private static SimulationSettings simSettings;
        private static bool thread;
        private static Force2 netForce;
        private static CollisionData collisionData;

        /// <summary>
        /// Starts the simulation.
        /// </summary>
        /// <param name="Settings">The settings for the simulation.</param>
        public void StartSimulation(SimulationSettings Settings)
        {
            simThread = new System.Threading.Thread(new ThreadStart(Simulate));
            simThread.Name = "Simulation Thread";
            simSettings = Settings;
            simThread.Start();
        }

        /// <summary>
        /// Runs the whole simulation.
        /// </summary>
        /// <param name="SimulationSettings">The simulation settings.</param>
        private static void Simulate()
        {
            SimulationForm simForm = new SimulationForm();
            simForm.FormClosing += new System.Windows.Forms.FormClosingEventHandler(EarlyTermination);

            Thread simFormThread = new Thread(new ParameterizedThreadStart(ShowSimForm));
            simFormThread.Name = "Simulation Form Thread";
            simFormThread.Start(simForm);

            int frameCount = 0;
            bool killed = false;
            System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();
            float simTime = 0;
            
            sw.Start();
            /*Continues looping until a kill condition is reached*/
            while (!killed)
            {
                frameCount++;
                simTime += simSettings.StepValue;

                /*Iterates over all the objects in the simulation*/
                for (int i = 0; i < simSettings.Objects.Count; i++)
                {
                    netForce = new Force2();

                    /* Determines if it would be appropriate to utilize multiple threads for this simulation.
                     * If multiple threads would be beneficial then the threads are set up else the force calculations are
                     * started immediately*/
                    if (simSettings.Force.Count > 20 && Environment.ProcessorCount > 1)
                    {
                        thread = true;
                        
                        Thread forceCalcThread = new Thread(new ParameterizedThreadStart(CalculateForce));
                        Thread gravityForceCalcThread = new Thread(new ParameterizedThreadStart(CalculateGravity));

                        forceCalcThread.Start(i);
                        if (simSettings.Objects.Count > 1)
                        {
                            gravityForceCalcThread.Start(i);
                            while (forceCalcThread.IsAlive && gravityForceCalcThread.IsAlive) { }
                        }
                        else
                        {
                            while (forceCalcThread.IsAlive) { }
                        }
                    }
                    else
                    {
                        thread = false;

                        CalculateForce(i);
                        CalculateGravity(i);
                    }

                    simSettings.Objects[i].OldPosition = simSettings.Objects[i].Position;
                    simSettings.Objects[i].OldVelocity = simSettings.Objects[i].Velocity;
                    CalculatePosition(netForce, i);

                    /*Collision Check
                     * First a check is made to determine if the simulation would benefit from multithreading.
                     * If it is then the threads are prepared, else the collision check is just done immediately
                     */ 
                    if (simSettings.Objects.Count > 10 && Environment.ProcessorCount>1)
                    {
                        thread = true;

                        int countForEach = simSettings.Objects.Count / Environment.ProcessorCount;
                        int leftOver = simSettings.Objects.Count - countForEach * (Environment.ProcessorCount - 1);

                        ThreadedLoopSettings[] collisionCalcSettings = new ThreadedLoopSettings[Environment.ProcessorCount];
                        Thread[] collisionThreads = new Thread[Environment.ProcessorCount];

                        /*Prepares the collision calculation data and starts the collision calculation threads*/
                        for (int n = 0; n < collisionCalcSettings.Length; n++)
                        {
                            collisionCalcSettings[n] = new ThreadedLoopSettings();
                            collisionCalcSettings[n].CurrentObjectIndex = i;
                            collisionCalcSettings[n].FirstObjectToCheck = n * countForEach;
                            
                            if (n + 1 != Environment.ProcessorCount)
                            {
                                collisionCalcSettings[n].LastObjectToCheck = n * countForEach + countForEach;
                            }
                            else
                            {
                                collisionCalcSettings[n].LastObjectToCheck = n * countForEach + leftOver;
                            }

                            collisionThreads[n] = new Thread(new ParameterizedThreadStart(CollisionCheck));
                            collisionThreads[n].Start(collisionCalcSettings[n]);
                        }

                        bool stillChecking = false;
                        /*Stalls while waiting for the collision calculations are finished up*/
                        while (true)
                        {
                            for (int n = 0; n < collisionThreads.Length; n++)
                            {
                                if (collisionThreads[n].IsAlive)
                                    stillChecking = true;
                            }

                            if (!stillChecking)
                            {
                                break;
                            }
                            stillChecking = false;
                        }
                    }
                    else
                    {
                        thread = false;

                        ThreadedLoopSettings calcSettings = new ThreadedLoopSettings();
                        calcSettings.CurrentObjectIndex = i;
                        calcSettings.FirstObjectToCheck = 0;
                        calcSettings.LastObjectToCheck = simSettings.Objects.Count - 1;
                        CollisionCheck(calcSettings);
                    }
                    

                    /*Now there is a check done for a kill condition*/                    
                    killed = CheckForKill(simSettings.Kill, simSettings.Objects[i], simTime, sw.Elapsed);
                    

                    /*Updates reports on objects*/
                    FillReport(collisionData.CollidedWith, collisionData.CollisionOccurred, collisionData.CollisionCount, i);
                }
            }
            sw.Stop();


            /*Generates the simulation report*/
            simForm.ReportGenerationStart();
            SimulationReport simReport = new SimulationReport();
            simReport.NumberOfForces = simSettings.Force.Count;
            simReport.NumberOfFramesCalculated = frameCount;
            simReport.NumberOfObjects = simSettings.Objects.Count;
            simReport.SimTime = simTime;
            simReport.TimeRequiredToSimulate = sw.Elapsed;
            GenerateReports(simSettings.Reports, simReport, simSettings.FilePath);

            simForm.SimOver();
        }

        private static void ShowSimForm(object simForm)
        {
            SimulationForm tempForm = (SimulationForm)simForm;
            tempForm.ShowDialog();
        }

        /// <summary>
        /// Checks for any collisions in the simulation.
        /// </summary>
        /// <param name="collisionCalculationSettings">The collision calculation settings.</param>
        private static void CollisionCheck(object collisionCalculationSettings)
        {
            ThreadedLoopSettings calcSettings = (ThreadedLoopSettings)collisionCalculationSettings;

            int i = calcSettings.CurrentObjectIndex;

            collisionData = new CollisionData();

            for (int n = calcSettings.FirstObjectToCheck; n < calcSettings.LastObjectToCheck && n != i; n++)
            {
                if (Vector2.DistanceBetweenVectors(simSettings.Objects[n].Position, simSettings.Objects[i].Position) == 0)
                {
                    collisionData.CollisionOccurred = true;
                    collisionData.CollidedWith.Add(simSettings.Objects[n].Name);
                    collisionData.CollisionCount++;

                    /*Find normal vector*/
                    Vector2 normalVector = simSettings.Objects[i].Velocity - simSettings.Objects[n].Velocity;
                    /*Find the unit vector*/
                    Vector2 unitVector = normalVector / Vector2.FindMagnitude(normalVector);
                    /*Find the unit tangent*/
                    Vector2 unitTangentVector = new Vector2(-unitVector.YValue, unitVector.XValue);

                    /*Normal numbers*/
                    double velocityOneNormal = Vector2.DotProduct(unitVector, simSettings.Objects[n].Velocity);
                    double velocityOneTangent = Vector2.DotProduct(unitTangentVector, simSettings.Objects[n].Velocity);

                    double velocityTwoNormal = Vector2.DotProduct(unitVector, simSettings.Objects[i].Velocity);
                    double velocityTwoTanget = Vector2.DotProduct(unitTangentVector, simSettings.Objects[i].Velocity);

                    /*Finding the normals after collision*/
                    double velocityOnePrimeNormal = (velocityOneNormal * (simSettings.Objects[n].Mass - simSettings.Objects[i].Mass) +
                        2 * simSettings.Objects[i].Mass * velocityTwoNormal) /
                        (simSettings.Objects[n].Mass + simSettings.Objects[i].Mass);

                    double velocityTwoPrimeNormal = (velocityTwoNormal * (simSettings.Objects[i].Mass - simSettings.Objects[n].Mass) +
                        2 * simSettings.Objects[n].Mass * velocityOneNormal) /
                        (simSettings.Objects[i].Mass + simSettings.Objects[n].Mass);

                    /*Finding the vectors after collision*/
                    Vector2 velocityOnePrimeNormalVector = velocityOnePrimeNormal * unitVector;
                    Vector2 velocityOnePrimeTangentVector = velocityOneTangent * unitTangentVector;

                    Vector2 velocityTwoPrimeNormalVector = velocityTwoPrimeNormal * unitVector;
                    Vector2 velocityTwoPrimeTangentVector = velocityTwoTanget * unitTangentVector;

                    /*Finally calculate the resulting velocity*/
                    simSettings.Objects[n].Velocity = velocityOnePrimeNormalVector + velocityOnePrimeTangentVector;
                    if (thread)
                    {
                        lock ((object)simSettings.Objects[i].Velocity)
                        {
                            simSettings.Objects[i].Velocity = velocityTwoPrimeNormalVector + velocityTwoPrimeTangentVector;
                        }
                    }
                    else
                    {
                        simSettings.Objects[i].Velocity = velocityTwoPrimeNormalVector + velocityTwoPrimeTangentVector;
                    }

                }
            }
        }

        /// <summary>
        /// Fills the report for the current object.
        /// </summary>
        /// <param name="collidedWith">The collided with.</param>
        /// <param name="collisionOccurred">if set to <c>true</c> [collision occurred].</param>
        /// <param name="collisionCount">The collision count.</param>
        /// <param name="i">The current object.</param>
        private static void FillReport(List<string> collidedWith, bool collisionOccurred, int collisionCount, int i)
        {
            for (int n = 0; n < simSettings.Reports.Count; n++)
            {
                if (simSettings.Reports[n].Settings.ObjectName == simSettings.Objects[i].Name)
                {
                    if (simSettings.Reports[n].Data.ObjectName != simSettings.Reports[n].Settings.ObjectName)
                    {
                        simSettings.Reports[n].Data.ObjectName = simSettings.Reports[n].Settings.ObjectName;
                    }
                    if (simSettings.Reports[n].Settings.RecordVelocity)
                    {
                        if (!simSettings.RunInLightMode)
                        {
                            simSettings.Reports[n].Data.Velocity.Add(simSettings.Objects[i].Velocity);
                        }
                        else
                        {
                            simSettings.Reports[n].Data.Velocity[0] = simSettings.Objects[i].Velocity;
                        }
                    }
                    if (simSettings.Reports[n].Settings.RecordPosition)
                    {
                        if (!simSettings.RunInLightMode)
                        {
                            simSettings.Reports[n].Data.Position.Add(simSettings.Objects[i].Position);
                        }
                        else
                        {
                            simSettings.Reports[n].Data.Position[0] = simSettings.Objects[i].Position;
                        }
                    }
                    if (simSettings.Reports[n].Settings.RecordCollisionCount)
                    {
                        if (collisionOccurred)
                            simSettings.Reports[n].Data.CollisionCount += collisionCount;
                    }
                    if (simSettings.Reports[n].Settings.RecordObjectsCollideAgainst)
                    {
                        if (collisionOccurred)
                        {
                            simSettings.Reports[n].Data.ObjectsCollidedWith.AddRange(collidedWith);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Calculates the net force acting upon the object, not including the gravitational pull of other objects.
        /// </summary>
        /// <param name="currentObjectIndex">Index of the current object.</param>
        private static void CalculateForce(object currentObjectIndex)
        {
            int i = (int)currentObjectIndex;

            Force2 currentForce = new Force2();
            /*Then tracks and checks to see if a force would act upon the object*/
            for (int n = 0; n < simSettings.Force.Count; n++)
            {
                if (simSettings.Force[n].Active)
                {
                    if (simSettings.Force[n].WorldForce)
                    {
                        currentForce.Acceleration += simSettings.Force[n].Acceleration;
                    }
                    else if (simSettings.Force[n].AffectedArea.XValue <= simSettings.Objects[i].Position.XValue &&
                        simSettings.Force[n].AffectedArea.YValue >= simSettings.Objects[i].Position.XValue)
                    {
                        currentForce.Acceleration += simSettings.Force[n].Acceleration;
                    }
                }
            }

            if (thread)
            {
                lock (netForce)
                {
                    netForce.Acceleration += currentForce.Acceleration;
                }
            }
            else
            {
                netForce.Acceleration += currentForce.Acceleration;
            }
        }

        /// <summary>
        /// Calculates the gravitational pull of of other objects on the object currently being worked upon.
        /// </summary>
        /// <param name="currentObjectIndex">Index of the current object.</param>
        private static void CalculateGravity(object currentObjectIndex)
        {
            int i = (int)currentObjectIndex;

            Force2 gravityForce = new Force2();
            /*Calculates the gravitational pull of the other object in the world*/
            for (int z = 0; z < simSettings.Objects.Count && z != i; z++)
            {
                double magnitudeOfGravity = (6.67 * Math.Pow(10, -11)) *
                    (simSettings.Objects[z].Mass / Math.Pow(
                    Vector2.DistanceBetweenVectors(simSettings.Objects[i].Position,
                    simSettings.Objects[z].Position), 2));
                double angleOfGravity = Vector2.FindAngleBetweenTwoVectors(simSettings.Objects[i].Position,
                    simSettings.Objects[z].Position);

                gravityForce.Acceleration += Vector2.CreateVector(magnitudeOfGravity, angleOfGravity);
            }

            if (thread)
            {
                lock (netForce)
                {
                    netForce.Acceleration += gravityForce.Acceleration;
                }
            }
            else
            {
                netForce.Acceleration += gravityForce.Acceleration;
            }

        }

        /// <summary>
        /// Calculates the position of an object based of a force acting upon the object.
        /// </summary>
        /// <param name="force">The force modifying the object.</param>
        /// <param name="currentObjectIndex">The index of the current object being working upon</param>
        private static void CalculatePosition(Force2 force, int currentObjectIndex)
        {
            simSettings.Objects[currentObjectIndex].Velocity += force.Acceleration * simSettings.StepValue;
            simSettings.Objects[currentObjectIndex].Position += simSettings.Objects[currentObjectIndex].Velocity;
        }

        /// <summary>
        /// Checks for kill condition.
        /// </summary>
        /// <param name="kill">The kill condition.</param>
        /// <param name="pObject">The object.</param>
        /// <param name="simTime">The simulation elapsed time.</param>
        /// <param name="realTime">The real elapsed time.</param>
        /// <returns></returns>
        private static bool CheckForKill(KillCondition kill, PhysicsObject2 pObject, float simTime, TimeSpan realTime)
        {
            if (kill.EndAfterRealWorldTimeElapse)
            {
                if (kill.RealWorldElapse <= realTime)
                    return true;
            }
            else if (kill.EndAfterSimulationElapsedTime)
            {
                /* Conversion is necessary because the step value is in seconds and there for the simulation time is
                 * calculated in seconds.*/
                float convertedToSecondsTime = 0;
                convertedToSecondsTime = (kill.SimulationElapsedTime.Days * 86400) +
                    (kill.SimulationElapsedTime.Hours * 3600) +
                    (kill.SimulationElapsedTime.Minutes * 60) +
                    kill.SimulationElapsedTime.Seconds +
                    (kill.SimulationElapsedTime.Milliseconds * .001f) +
                    (kill.SimulationElapsedTime.Ticks * .00000001f);

                if (convertedToSecondsTime <= simTime)
                    return true;
            }

            if (simSettings.Kill.NameOfObjectBeingMonitored == pObject.Name)
            {
                if (kill.EndAtVector)
                {
                    if (kill.EndVector.XValue <= pObject.Position.XValue &&
                        kill.EndVector.YValue <= pObject.Position.XValue)
                        return true;
                }
                else if (kill.EndAtVelocity)
                {
                    if (kill.EndVelocity == pObject.Velocity)
                        return true;
                }
                else if (kill.EndAtXCoord)
                {
                    if (pObject.Position.XValue >= kill.EndXCoord && pObject.OldPosition.XValue < kill.EndXCoord)
                        return true;
                    else if (pObject.Position.XValue <= kill.EndXCoord && pObject.OldPosition.XValue > kill.EndXCoord)
                        return true;
                }
                else if (kill.EndAtYCoord)
                {
                    if (pObject.Position.YValue >= kill.EndYCoord && pObject.OldPosition.YValue < kill.EndYCoord)
                        return true;
                    else if (pObject.Position.YValue <= kill.EndYCoord && pObject.OldPosition.YValue > kill.EndYCoord)
                        return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Generates the reports for the simulation.
        /// </summary>
        /// <param name="reports">The reports.</param>
        /// <param name="simReport">The simulation report.</param>
        /// <param name="filePath">The file path.</param>
        private static void GenerateReports(List<Report> reports, SimulationReport simReport, string filePath)
        {
            ReportGenerator reportGen = new ReportGenerator();
            reportGen.GenerateCompleteReport(reports, simReport, filePath);
        }

        /// <summary>
        /// Terminates the thread when the form is closed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        static void EarlyTermination(object sender, System.Windows.Forms.FormClosingEventArgs e)
        {
            if (simThread.ThreadState == System.Threading.ThreadState.Running)
                simThread.Abort();
        }
    }
}
