﻿#region License Informtion
/* This program simulates objects path of motion.
 * Copyright (C) 2010  Timothy Eck II
 * 
 * Physics Simulator is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; version 2 of the License.
 * 
 * Physics Simulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PhysicsSimulator.Engine
{
    /// <summary>
    ///  A double precision value type for a vector containing two components.
    /// </summary>
    public struct Vector2
    {
        private double _xValue;
        private double _yValue;

        /// <summary>
        /// This property returns the x component of the vector
        /// </summary>
        /// <value>The X value.</value>
        public double XValue
        {
            get { return _xValue; }
            set { _xValue = value; }
        }

        /// <summary>
        /// This property returns the y component of the vector
        /// </summary>
        /// <value>The Y value.</value>
        public double YValue
        {
            get { return _yValue; }
            set { _yValue = value; }
        }

        /// <summary>
        /// Returns a vector that is equal to (0,0)
        /// </summary>
        public static Vector2 Zero { get { return new Vector2(0); } }
        
        /// <summary>
        /// Initializes a new instance of the <see cref="Vector2"/> struct.
        /// </summary>
        /// <param name="xValue">The x value.</param>
        /// <param name="yValue">The y value.</param>
        public Vector2(double xValue = 0, double yValue = 0)
        {
            _xValue = xValue;
            _yValue = yValue;
        }

        /// <summary>
        /// Finds the magnitude of a vector.
        /// </summary>
        /// <param name="vector">The vector to find the magnitude of.</param>
        /// <returns>Returns the magnitude of the vector.</returns>
        public static double FindMagnitude(Vector2 vector)
        {
            return Math.Sqrt(Math.Pow(vector.XValue, 2) + Math.Pow(vector.YValue, 2));
        }

        /// <summary>
        /// Finds the angle of the vector.
        /// </summary>
        /// <param name="vector">The vector to find the angle of.</param>
        /// <returns>
        /// Returns the angle of the vector in radians.
        /// </returns>
        public static double FindAngle(Vector2 vector)
        {
            return Math.Atan(vector.YValue / vector.XValue);
        }

        /// <summary>
        /// Distances the between vectors.
        /// </summary>
        /// <param name="firstVector">The first vector.</param>
        /// <param name="secondVector">The second vector.</param>
        /// <returns>Returns the distance between the two vectors.</returns>
        public static double DistanceBetweenVectors(Vector2 firstVector, Vector2 secondVector)
        {
            return Math.Sqrt(Math.Pow((secondVector.XValue-firstVector.XValue),2)+
                Math.Pow((secondVector.YValue-firstVector.YValue),2));
        }

        /// <summary>
        /// Finds the angle between two vectors.
        /// </summary>
        /// <param name="firstVector">The first vector.</param>
        /// <param name="secondVector">The second vector.</param>
        /// <returns>Returns the angle in radians.</returns>
        public static double FindAngleBetweenTwoVectors(Vector2 firstVector, Vector2 secondVector)
        {
            return Math.Atan(
                (secondVector.YValue - firstVector.YValue) /
                (secondVector.XValue - firstVector.XValue));
        }

        /// <summary>
        /// Creates a vector from the magnitude and angle.
        /// </summary>
        /// <param name="magnitude">The magnitude.</param>
        /// <param name="angle">The angle in radians.</param>
        /// <returns>Returns the vector of the magnitude and angle.</returns>
        public static Vector2 CreateVector(double magnitude, double angle)
        {
            Vector2 tempVec = new Vector2();
            tempVec.YValue = magnitude * Math.Sin(angle);
            tempVec.YValue = magnitude * Math.Cos(angle);
            return tempVec;
        }

        public static double DotProduct(Vector2 firstVector, Vector2 secondVector)
        {
            return firstVector.XValue * secondVector.XValue + firstVector.YValue * secondVector.YValue;
        }

        public override string ToString()
        {
            return "(" + _xValue + "," + _yValue + ")";
        }

        public override bool Equals(object obj)
        {
            if ((object)this == obj)
                return true;
            return false;
        }

        #region Custom operators
        public static Vector2 operator *(Vector2 lhs, double rhs)
        {
            return rhs * lhs;
        }
        public static Vector2 operator *(double lhs, Vector2 rhs)
        {
            rhs.XValue *= lhs;
            rhs.YValue *= lhs;
            return rhs;
        }

        public static Vector2 operator /(Vector2 lhs, double rhs)
        {
            return rhs / lhs;
        }

        public static Vector2 operator /(double lhs, Vector2 rhs)
        {
            rhs.XValue /= lhs;
            rhs.YValue /= lhs;
            return rhs;
        }

        public static Vector2 operator +(double lhs, Vector2 rhs)
        {
            return rhs + lhs;
        }

        public static Vector2 operator +(Vector2 lhs, double rhs)
        {
            lhs.XValue += rhs;
            lhs.YValue += rhs;
            return lhs;
        }

        public static Vector2 operator +(Vector2 lhs, Vector2 rhs)
        {
            lhs.XValue += rhs.XValue;
            lhs.YValue += rhs.YValue;
            return lhs;
        }

        public static Vector2 operator -(Vector2 lhs, Vector2 rhs)
        {
            lhs.XValue -= rhs.XValue;
            lhs.YValue -= rhs.YValue;
            return lhs;
        }

        public static bool operator ==(Vector2 lhs, Vector2 rhs)
        {
            if ((object)lhs == null && (object)rhs == null)
                return true;
            else if ((object)lhs == null || (object)rhs == null)
                return false;

            if (lhs.XValue == rhs.XValue && lhs.YValue == rhs.YValue)
                return true;
            else return false;
        }

        public static bool operator !=(Vector2 lhs, Vector2 rhs)
        {
            if (lhs.XValue == rhs.XValue && lhs.YValue == rhs.YValue)
                return false;
            else return true;
        }
        
        public static explicit operator Vector2(int value)
        {
            return new Vector2(value);
        }
        #endregion
    }
}
