﻿#region License Informtion
/* This program simulates objects path of motion.
 * Copyright (C) 2010  Timothy Eck II
 * 
 * Physics Simulator is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; version 2 of the License.
 * 
 * Physics Simulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
#endregion
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using PhysicsSimulator.Engine;

namespace PhysicsSimulator.Reports
{
    /// <summary>
    /// Utility class used to generate reports.
    /// </summary>
    public class ReportGenerator
    {
        private string[] reportComponents = new string[4];

        #region Generation Methods
        /// <summary>
        /// Generates multiple partial reports.
        /// </summary>
        /// <param name="reports">The reports.</param>
        /// <param name="filePath">The file path.</param>
        /// <param name="fileName">Name of the file.</param>
        public void GeneratePartialReport(List<Report> reports, string filePath)
        {
            /* 0 = header and simulation information
                 * 1 = collision information
                 * 2 = velocity information
                 * 3 = position information */

            /*Threading check*/
            bool thread = false;
            foreach (Report report in reports)
            {
                if (report.Data.Position.Count > 1000 || report.Data.Velocity.Count > 1000
                    || report.Data.CollisionCount > 1000 && Environment.ProcessorCount > 1)
                    thread = true;
            }

            reportComponents[0] += "Report Generated on :," + DateTime.Now + "\n";
            
            if (thread)
            {
                Thread collisionDataThread = new Thread(new ParameterizedThreadStart(BuildCollision));
                collisionDataThread.Name = "Collision Data Thread";

                Thread velocityDataThread = new Thread(new ParameterizedThreadStart(BuildVelocity));
                velocityDataThread.Name = "Velocity Data Thread";

                Thread positionDataThread = new Thread(new ParameterizedThreadStart(BuildPosition));
                positionDataThread.Name = "Position Data Thread";

                collisionDataThread.Start(reports);
                velocityDataThread.Start(reports);
                positionDataThread.Start(reports);

                while (collisionDataThread.IsAlive || positionDataThread.IsAlive || velocityDataThread.IsAlive) { }
            }
            else
            {
                BuildCollision(reports);
                BuildVelocity(reports);
                BuildPosition(reports);
            }

            WriteToFile(filePath);
        }
        
        /// <summary>
        /// Generates a single partial report.
        /// </summary>
        /// <param name="report">The report.</param>
        /// <param name="filePath">The file path.</param>
        /// <param name="fileName">Name of the file.</param>
        public void GeneratePartialReport(Report report, string filePath)
        {
            GeneratePartialReport(new List<Report>() { report }, filePath);
        }

        /// <summary>
        /// Generates multiple complete reports.
        /// </summary>
        /// <param name="reports">The reports.</param>
        /// <param name="filePath">The file path.</param>
        /// <param name="fileName">Name of the file.</param>
        public void GenerateCompleteReport(List<Report> reports, SimulationReport simReport, string filePath)
        {

            /* 0 = header and simulation information
                 * 1 = collision information
                 * 2 = velocity information
                 * 3 = position information */

            /*Threading check*/
            bool thread = false;
            foreach (Report report in reports)
            {
                if (report.Data.Position.Count > 1000 || report.Data.Velocity.Count > 1000
                    || report.Data.CollisionCount > 1000 && Environment.ProcessorCount > 1)
                    thread = true;
            }

            reportComponents[0] += "Report Generated on :," + DateTime.Now + "\n";

            /*Simulation Data*/
            reportComponents[0] += "Number of objects:," + simReport.NumberOfObjects + "\n";
            reportComponents[0] += "Number of forces:," + simReport.NumberOfForces + "\n";
            reportComponents[0] += "Time required to simulate:," + simReport.TimeRequiredToSimulate + "\n";
            reportComponents[0] += "Time simulation elapse time:," + simReport.SimTime + "\n";
            reportComponents[0] += "Frames calculated:," + simReport.NumberOfFramesCalculated + "\n";

            if (thread)
            {
                Thread collisionDataThread = new Thread(new ParameterizedThreadStart(BuildCollision));
                collisionDataThread.Name = "Collision Data Thread";

                Thread velocityDataThread = new Thread(new ParameterizedThreadStart(BuildVelocity));
                velocityDataThread.Name = "Velocity Data Thread";

                Thread positionDataThread = new Thread(new ParameterizedThreadStart(BuildPosition));
                positionDataThread.Name = "Position Data Thread";

                collisionDataThread.Start(reports);
                velocityDataThread.Start(reports);
                positionDataThread.Start(reports);

                while (collisionDataThread.IsAlive || positionDataThread.IsAlive || velocityDataThread.IsAlive) { }
            }
            else
            {
                BuildCollision(reports);
                BuildVelocity(reports);
                BuildPosition(reports);
            }

            WriteToFile(filePath);
        }

        /// <summary>
        /// Generates a single complete report.
        /// </summary>
        /// <param name="report">The report.</param>
        /// <param name="filePath">The file path.</param>
        /// <param name="fileName">Name of the file.</param>
        public void GenerateCompleteReport(Report report, SimulationReport simReport, string filePath)
        {
            GenerateCompleteReport(new List<Report>() { report }, simReport, filePath);
        }
        #endregion

        #region Helper Methods
        /// <summary>
        /// Builds the collision data string.
        /// </summary>
        /// <param name="parameters">The parameters.</param>
        private void BuildCollision(object parameters)
        {
            List<Report> reports = (List<Report>)parameters;

            reportComponents[1] += "Collision data\n";
            foreach (Report report in reports)
            {
                if (report.Settings.RecordCollisionCount || report.Settings.RecordObjectsCollideAgainst)
                {
                    string collisionInfo = "";
                    collisionInfo += report.Data.ObjectName + "\n";

                    collisionInfo += report.Data.CollisionCount + "\n";
                    collisionInfo += "Objects Collided With:,";

                    if (report.Data.ObjectsCollidedWith.Count > 0)

                        foreach (string name in report.Data.ObjectsCollidedWith)
                        {
                            collisionInfo += name + ",";
                        }

                    else
                    {
                        collisionInfo += "none";
                    }

                    reportComponents[1] += collisionInfo + "\n";
                }
                else
                {
                    reportComponents[1] += "N/A\n";
                }
            }
        }

        /// <summary>
        /// Builds the velocity data string.
        /// </summary>
        /// <param name="parameters">The parameters.</param>
        private void BuildVelocity(object parameters)
        {
            List<Report> reports = (List<Report>)parameters;

            reportComponents[2] += "Velocity Data\n";

            foreach (Report report in reports)
            {
                if (report.Settings.RecordPosition)
                {
                    string velocityInfo = "";
                    velocityInfo += report.Data.ObjectName + "\n";

                    string[] velocityComponents = new string[2];
                    for (int i = 0; i < report.Data.Velocity.Count; i++)
                    {
                        velocityComponents[0] += report.Data.Velocity[i].XValue + ",";
                        velocityComponents[1] += report.Data.Velocity[i].YValue + ",";
                    }

                    velocityInfo += "X:," + velocityComponents[0] + "\n";
                    velocityInfo += "Y:," + velocityComponents[1] + "\n";

                    reportComponents[2] += velocityInfo + "\n";
                }
            }
        }

        /// <summary>
        /// Builds the position data string.
        /// </summary>
        /// <param name="parameters">The parameters.</param>
        private void BuildPosition(object parameters)
        {
            List<Report> reports = (List<Report>)parameters;
            reportComponents[3] += "Position Data\n";

            foreach (Report report in reports)
            {
                if (report.Settings.RecordPosition)
                {
                    string positionInfo = "";
                    positionInfo += report.Data.ObjectName + "\n";

                    string[] positionComponents = new string[2];

                    for (int i = 0; i < report.Data.Position.Count; i++)
                    {
                        positionComponents[0] += report.Data.Position[i].XValue + ",";
                        positionComponents[1] += report.Data.Position[i].YValue + ",";
                    }

                    positionInfo += "X:," + positionComponents[0] + "\n";
                    positionInfo += "Y:," + positionComponents[1] + "\n";

                    reportComponents[3] += positionInfo + "\n";
                }
            }
        }

        /// <summary>
        /// Writes the report data to file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        private void WriteToFile(string filePath)
        {
            /* Error Code 0001 = There is an error encountered while trying to gain access to the file or writing to the file.
             *                   Make sure that no other applications are using the file.
             * Error Code 0002 = There was an unknown error encountered while trying to gain access to the file or writing to the file.
             *                   
             */
            try
            {
                FileStream fs = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.Write);
                StreamWriter writer = new StreamWriter(fs);
                writer.Write(reportComponents[0]);
                writer.Write(reportComponents[1]);
                writer.Write(reportComponents[2]);
                writer.Write(reportComponents[3]);
                writer.Close();
            }
            catch (IOException e)
            {
                System.Windows.Forms.DialogResult result = System.Windows.Forms.MessageBox.Show(
                    "Error Code: 0001\nThere was an error creating or writing to" +
                    "the file and the report can not be created.", "Error Code 0001",
                    System.Windows.Forms.MessageBoxButtons.RetryCancel);

                if (result == System.Windows.Forms.DialogResult.Retry)
                    WriteToFile(filePath);
            }
            catch (Exception e)
            {
                System.Windows.Forms.DialogResult result = System.Windows.Forms.MessageBox.Show(
                    "Error Code: 0002\nThere was an unknown error creating or writing to" +
                    "the file and the report can not be created.", "Error Code 0002",
                    System.Windows.Forms.MessageBoxButtons.RetryCancel);

                if (result == System.Windows.Forms.DialogResult.Retry)
                    WriteToFile(filePath);
            }
        }
        #endregion
    }
}
